﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Reversi
{
    class ReversiSilnik
    {
        //gracze: 1, 2, 0 - puste

        public int SzerokośćPlanszy { get; private set; }
        public int WysokośćPlanszy { get; private set; }

        private int[,] plansza;
        public int NumerGraczaWykonującegoNastępnyRuch { get; private set; } = 1;

        private static int numerPrzeciwnika(int numerGracza)
        {
            return (numerGracza == 1) ? 2 : 1;
        }

        private bool czyWspółrzędnePolaPrawidłowe(int poziomo, int pionowo)
        {
            return
                poziomo >= 0 && poziomo < SzerokośćPlanszy &&
                pionowo >= 0 && pionowo < WysokośćPlanszy;
        }

        public int PobierzStanPola(int poziomo, int pionowo)
        {
            if (!czyWspółrzędnePolaPrawidłowe(poziomo, pionowo))
                throw new ArgumentException("Nieprawidłowe współrzędne pola");
            return plansza[poziomo, pionowo];
        }

        private void czyśćPlanszę()
        {
            for(int poziomo = 0; poziomo < SzerokośćPlanszy; poziomo++)
                for(int pionowo = 0; pionowo < WysokośćPlanszy; pionowo++)
                {
                    plansza[poziomo, pionowo] = 0;
                }

            int środekSzer = SzerokośćPlanszy / 2;
            int środekWys = WysokośćPlanszy / 2;
            plansza[środekSzer - 1, środekWys - 1] = plansza[środekSzer, środekWys] = 1;
            plansza[środekSzer, środekWys - 1] = plansza[środekSzer - 1, środekWys] = 2;
        }

        public ReversiSilnik(int numerGraczaRozpoczynającego = 1, int szerokośćPlanszy = 8, int wysokośćPlanszy = 8)
        {
            if (numerGraczaRozpoczynającego < 1 || numerGraczaRozpoczynającego > 2)
                throw new ArgumentException("Niepoprawny numer gracza");

            SzerokośćPlanszy = szerokośćPlanszy;
            WysokośćPlanszy = wysokośćPlanszy;
            plansza = new int[SzerokośćPlanszy, WysokośćPlanszy];

            czyśćPlanszę(); //bez sensu

            NumerGraczaWykonującegoNastępnyRuch = numerGraczaRozpoczynającego;
        }

        public int PołóżKamień(int poziomo, int pionowo) //-1 - nie położyć kamienia
        {
            if (!czyWspółrzędnePolaPrawidłowe(poziomo, pionowo))
                throw new Exception("Nieprawidłowe współrzędne pola planszy");

            if (plansza[poziomo, pionowo] != 0)
                //return -1;
                throw new Exception("Pole nie jest puste");

            List<Tuple<int,int>> przejmowanePola = new List<Tuple<int, int>>();

            for (int kierunekPoziomo = -1; kierunekPoziomo <= 1; kierunekPoziomo++)
                for (int kierunekPionowo = -1; kierunekPionowo <= 1; kierunekPionowo++)
                {
                    if (kierunekPoziomo == 0 && kierunekPionowo == 0) continue;

                    List<Tuple<int, int>> przejmowanePolaWKierunku = new List<Tuple<int, int>>();                    

                    int i = poziomo;
                    int j = pionowo;
                    //bool znalezionyKamieńPrzeciwnika = false;
                    bool znalezionyKamieńGraczaWykonująceRuch = false;
                    bool znalezionePustePole = false;
                    bool osiągniętaKrawędźPlanszy = false;
                    do
                    {
                        i += kierunekPoziomo;
                        j += kierunekPionowo;
                        if (!czyWspółrzędnePolaPrawidłowe(i, j))
                        {
                            osiągniętaKrawędźPlanszy = true;
                            break;
                        }

                        if (plansza[i, j] == NumerGraczaWykonującegoNastępnyRuch)
                            znalezionyKamieńGraczaWykonująceRuch = true;
                        if (plansza[i, j] == 0) znalezionePustePole = true;
                        if (plansza[i, j] == numerPrzeciwnika(NumerGraczaWykonującegoNastępnyRuch))
                        {
                            przejmowanePolaWKierunku.Add(new Tuple<int, int>(i, j));
                            //znalezionyKamieńPrzeciwnika = true;
                        }
                    }
                    while (!znalezionyKamieńGraczaWykonująceRuch && !znalezionePustePole && !osiągniętaKrawędźPlanszy);

                    if (!znalezionePustePole && !osiągniętaKrawędźPlanszy) przejmowanePola.AddRange(przejmowanePolaWKierunku);
                }

            if (przejmowanePola.Count == 0) return 0;
            else
            {
                plansza[poziomo, pionowo] = NumerGraczaWykonującegoNastępnyRuch;
                foreach (Tuple<int, int> pole in przejmowanePola)
                    plansza[pole.Item1, pole.Item2] = NumerGraczaWykonującegoNastępnyRuch;
                NumerGraczaWykonującegoNastępnyRuch = numerPrzeciwnika(NumerGraczaWykonującegoNastępnyRuch);
                return przejmowanePola.Count;
            }            
        }
    }
}
